==> Iuno-3.2.2.11072.tar.gz: Iuno-3.2.2.11072/Iuno/template_editor/admin.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# $Id: admin.py 10982 2018-04-16 08:14:06Z Lavender $
#
# Copyright (c) 2017 Nuwa Information Co., Ltd, All Rights Reserved.
#
# Licensed under the Proprietary License,
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at our web site.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# $Author: Lavender $
# $Date: 2018-04-16 16:14:06 +0800 (週一, 16 四月 2018) $
# $Revision: 10982 $

import os
import datetime
import hashlib

from functools import update_wrapper
from django.contrib import admin
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.contrib.admin.templatetags.admin_static import static
from django.contrib.auth.signals import user_logged_in
from django.forms import Media
from django.db import transaction
from django.conf.urls import url
from django.http import HttpResponseRedirect
from django.contrib import messages
from django.core.urlresolvers import reverse

# 介面拖拉上傳

# ------------------------------------------------------------------------------
from Iuno.template_editor.models import Static, Template, Version, FOLDER, FILE
from Iuno.template_editor import BACKUP_FILE_DIR

TEXT_SUB_FILE_NAME = [
    'txt', 'css', 'js', 'html', 'htm',
]

IMAGE_SUB_FILE_NAME = [
    'jpg', 'png', 'jpeg', 'bmp', 'gif', 'tif',
]


class FolderFrom(forms.ModelForm):
    class Meta(object):
        model = Static
        exclude = ('parent', 'type', 'name')

class TextFrom(forms.ModelForm):
    content = forms.CharField(
        label=_('Content'), widget=forms.Textarea, required=False)

    class Meta(object):
        model = Static
        exclude = ('parent', 'type', 'name')

class ImageFrom(forms.ModelForm):
    image = forms.ImageField(label=_('Image'), required=False)

    class Meta(object):
        model = Static
        exclude = ('parent', 'type', 'name')

def createVersion(model, node, conflict=False):
    fileName = node.name.split('.')[0]
    subFileName = node.subFileName

    fullPath = node.absolutePath
    path = node.path

    with open(fullPath, 'rb') as f:
        data = f.read()

    dirName = hashlib.md5(path).hexdigest()
    checksum = hashlib.md5(data).hexdigest()

    createFile = False

    if Version.objects.filter(dirName=dirName).exists():
        # conflict or not
        preVersion = node.version
        maxVersion = preVersion.number

        if subFileName:
            backupName = "%s_%d.%s" % (fileName, maxVersion + 1, subFileName)
        else:
            backupName = "%s_%d" % (fileName, maxVersion + 1)

        if not preVersion.checksum == checksum:
            if not maxVersion == 0 or not conflict:
                version = Version.objects.create(
                    dirName=dirName,
                    checksum=checksum,
                    number=maxVersion + 1,
                    fileName=backupName,
                    conflict=conflict,
                    preVersion=preVersion,
                )
                preVersion.nextVersion = version
                preVersion.save()
            else:
                preVersion.checksum = checksum
                preVersion.save()
                if subFileName:
                    backupName = "%s_%d.%s" % (
                        fileName, maxVersion, subFileName)
                else:
                    backupName = "%s_%d" % (fileName, maxVersion)

                version = preVersion

            createFile = True
        else:
            version = preVersion
    else:
        maxVersion = 0
        if subFileName:
            backupName = "%s_%d.%s" % (fileName, maxVersion, subFileName)
        else:
            backupName = "%s_%d" % (fileName, maxVersion)

        version = Version.objects.create(
            dirName=dirName,
            checksum=checksum,
            number=maxVersion,
            fileName=backupName,
        )
        createFile = True 

    if createFile:
        dirPath = os.path.join(BACKUP_FILE_DIR, dirName)
        if not os.path.isdir(dirPath):
            os.mkdir(dirPath)

        backupPath = os.path.join(dirPath, backupName)

        with open(backupPath, 'wb') as f:
            f.write(data)

    return version

@transaction.atomic
def checkVersion(model):
    conflictNodeList = []
    for node in model.objects.filter(type=FILE):
        version = createVersion(model, node, conflict=True)

        path = node.path
        dirName = hashlib.md5(path).hexdigest()

        for version in Version.objects.filter(
            dirName=dirName, conflict=True):
            conflictNodeList.append((node, version))

    return conflictNodeList

def loginCheck(sender, user, request, **kwargs):
    conflictNodeList = checkVersion(Static) + checkVersion(Template)
    for ele in conflictNodeList:
        node = ele[0]
        version = ele[1]

        msg = _("Version %(version)d "
                "conflict Version %(preVersion)d !!: %(path)s") % {
                'version': version.number, 
                'preVersion': version.preVersion.number,
                'path': node.path,
            }

        messages.warning(request, msg)

user_logged_in.connect(loginCheck)

class BaseFileAdmin(admin.ModelAdmin):
    change_list_template = 'template_editor/change_list.html'
    change_form_template = 'template_editor/change_form.html'

    def get_urls(self):
        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name
        
        urlpatterns = [
            url(r'^version/(?P<id>.+)/removeConflict/$', 
                wrap(self.removeConflictView), 
                name="%s_%s_removeConflict" % info),
        ] + super(BaseFileAdmin, self).get_urls()

        return urlpatterns

    def removeConflictView(self, request, extra_context=None, **kwargs):
        version = Version.objects.get(id=kwargs['id'])
        version.conflict = False
        version.save()

        info = self.model._meta.app_label, self.model._meta.model_name
        url = reverse('admin:%s_%s_changelist' % info)
        messages.success(request, _('Conflict solved.'))
        return HttpResponseRedirect(url)


    def get_form(self, request, obj=None, **kwargs):
        if not obj:
            # add Form 新增能夠上傳檔案來新增 static 檔案
            return super(BaseFileAdmin, self).get_form(request, obj, **kwargs)

        # is folder
        if obj.type == FOLDER:
            return FolderFrom

        # is file
        versionNum = request.GET.get('version', None)
        if versionNum:
            version = obj.getVersion(versionNumber=versionNum)
        else:
            version = obj.version

        fullPath = version.absoluteBackupPath
        subFileName = obj.subFileName

        if subFileName in TEXT_SUB_FILE_NAME:
            form = TextFrom
            with open(fullPath, 'rb') as f:
                data = f.read()
            form.base_fields['content'].initial = data
        elif subFileName in IMAGE_SUB_FILE_NAME:
            form = ImageFrom
        else:
            form = FolderFrom

        return form

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = {}
        obj = self.model.objects.get(id=object_id)

        versionNum = request.GET.get('version', None)
        if versionNum:
            extra_context['version'] = obj.getVersion(versionNumber=versionNum)
        else:
            extra_context['version'] = obj.version

        subFileName = obj.subFileName

        if subFileName in TEXT_SUB_FILE_NAME:
            extra_context['isImg'] = False
        elif subFileName in IMAGE_SUB_FILE_NAME:
            extra_context['isImg'] = True
        else:
            extra_context['isImg'] = False

        extra_context['moduleName'] = obj._meta.verbose_name_plural

        return super(BaseFileAdmin, self).change_view(
            request, object_id, form_url=form_url, extra_context=extra_context)

    def response_change(self, request, obj):
        versionNum = request.GET.get('version', None)
        if versionNum:
            version = obj.getVersion(versionNumber=versionNum)
        else:
            version = obj.version

        if obj.type == FILE:
            subFileName = obj.subFileName

            if subFileName in TEXT_SUB_FILE_NAME:
                # 能支援上傳修改檔案
                content = request.POST.get('content')
                
                fullPath = obj.absolutePath
                with open(fullPath, 'wb') as f:
                    data = f.write(content.encode('utf-8'))
            elif subFileName in IMAGE_SUB_FILE_NAME:
                # 圖片轉檔
                image = request.FILES.get('image')
                fullPath = obj.absolutePath

                if image:
                    data = image.read()
                    image.close()

                    with open(fullPath, 'wb') as f:
                        data = f.write(data)
                elif not obj.version == version:
                    backupPath = version.absoluteBackupPath
                    with open(backupPath, 'rb') as f:
                        data = f.read()
                    fullPath = obj.absolutePath
                    with open(fullPath, 'wb') as f:
                        data = f.write(data)
                else:
                    pass
            
            version = createVersion(self.model, obj)

        return super(BaseFileAdmin, self).response_change(request, obj)

    def changelist_view(self, request, extra_context=None):
        conflictNodeList = checkVersion(self.model)

        roots = self.model.objects.filter(level=0)
         
        extra_context = {
            'roots': roots,
            'conflictNodeList': conflictNodeList,
        }
        
        return super(BaseFileAdmin, self).changelist_view(
            request,  extra_context=extra_context)

class StaticAdmin(BaseFileAdmin):
    pass

class TemplateAdmin(BaseFileAdmin):
    pass

admin.site.register(Static, StaticAdmin)
admin.site.register(Template, TemplateAdmin)

==> Iuno-3.2.3.11169.tar.gz: Iuno-3.2.3.11169/Iuno/template_editor/admin.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# $Id: admin.py 10982 2018-04-16 08:14:06Z Lavender $
#
# Copyright (c) 2017 Nuwa Information Co., Ltd, All Rights Reserved.
#
# Licensed under the Proprietary License,
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at our web site.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# $Author: Lavender $
# $Date: 2018-04-16 16:14:06 +0800 (週一, 16 四月 2018) $
# $Revision: 10982 $

import os
import datetime
import hashlib

from functools import update_wrapper
from django.contrib import admin
from django import forms
from django.conf import settings
from django.utils.html import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.contrib.admin.templatetags.admin_static import static
from django.contrib.auth.signals import user_logged_in
from django.forms import Media
from django.db import transaction
from django.conf.urls import url
from django.http import HttpResponseRedirect
from django.contrib import messages
from django.core.urlresolvers import reverse

# 介面拖拉上傳

# ------------------------------------------------------------------------------
from Iuno.template_editor.models import Static, Template, Version, FOLDER, FILE
from Iuno.template_editor import BACKUP_FILE_DIR

TEXT_SUB_FILE_NAME = [
    'txt', 'css', 'js', 'html', 'htm',
]

IMAGE_SUB_FILE_NAME = [
    'jpg', 'png', 'jpeg', 'bmp', 'gif', 'tif',
]


class FolderFrom(forms.ModelForm):
    class Meta(object):
        model = Static
        exclude = ('parent', 'type', 'name')

class TextFrom(forms.ModelForm):
    content = forms.CharField(
        label=_('Content'), widget=forms.Textarea, required=False)

    class Meta(object):
        model = Static
        exclude = ('parent', 'type', 'name')

class ImageFrom(forms.ModelForm):
    image = forms.ImageField(label=_('Image'), required=False)

    class Meta(object):
        model = Static
        exclude = ('parent', 'type', 'name')

def createVersion(model, node, conflict=False):
    fileName = node.name.split('.')[0]
    subFileName = node.subFileName

    fullPath = node.absolutePath
    path = node.path

    with open(fullPath, 'rb') as f:
        data = f.read()

    dirName = hashlib.md5(path).hexdigest()
    checksum = hashlib.md5(data).hexdigest()

    createFile = False

    if Version.objects.filter(dirName=dirName).exists():
        # conflict or not
        preVersion = node.version
        maxVersion = preVersion.number

        if subFileName:
            backupName = "%s_%d.%s" % (fileName, maxVersion + 1, subFileName)
        else:
            backupName = "%s_%d" % (fileName, maxVersion + 1)

        if not preVersion.checksum == checksum:
            if not maxVersion == 0 or not conflict:
                version = Version.objects.create(
                    dirName=dirName,
                    checksum=checksum,
                    number=maxVersion + 1,
                    fileName=backupName,
                    conflict=conflict,
                    preVersion=preVersion,
                )
                preVersion.nextVersion = version
                preVersion.save()
            else:
                preVersion.checksum = checksum
                preVersion.save()
                if subFileName:
                    backupName = "%s_%d.%s" % (
                        fileName, maxVersion, subFileName)
                else:
                    backupName = "%s_%d" % (fileName, maxVersion)

                version = preVersion

            createFile = True
        else:
            version = preVersion
    else:
        maxVersion = 0
        if subFileName:
            backupName = "%s_%d.%s" % (fileName, maxVersion, subFileName)
        else:
            backupName = "%s_%d" % (fileName, maxVersion)

        version = Version.objects.create(
            dirName=dirName,
            checksum=checksum,
            number=maxVersion,
            fileName=backupName,
        )
        createFile = True 

    if createFile:
        dirPath = os.path.join(BACKUP_FILE_DIR, dirName)
        if not os.path.isdir(dirPath):
            os.mkdir(dirPath)

        backupPath = os.path.join(dirPath, backupName)

        with open(backupPath, 'wb') as f:
            f.write(data)

    return version

@transaction.atomic
def checkVersion(model):
    conflictNodeList = []
    for node in model.objects.filter(type=FILE):
        version = createVersion(model, node, conflict=True)

        path = node.path
        dirName = hashlib.md5(path).hexdigest()

        for version in Version.objects.filter(
            dirName=dirName, conflict=True):
            conflictNodeList.append((node, version))

    return conflictNodeList

def loginCheck(sender, user, request, **kwargs):
    def createMessage(ele, model):
        node = ele[0]
        version = ele[1]

        info = model._meta.app_label, model._meta.model_name
        url = reverse('admin:%s_%s_change' % info, args=[node.id,])

        msg = _("Version %(version)d "
                "conflict Version %(preVersion)d !!: "
                "<a href='%(url)s'>%(path)s</a>") % {
                'version': version.number, 
                'preVersion': version.preVersion.number,
                'url': url,
                'path': node.path,
            }
        return mark_safe(msg)

    conflictNodeList = checkVersion(Static)
    for ele in conflictNodeList:
        msg = createMessage(ele, Static)
        messages.warning(request, msg)

    conflictNodeList = checkVersion(Template)
    for ele in conflictNodeList:
        msg = createMessage(ele, Template)
        messages.warning(request, msg)

user_logged_in.connect(loginCheck)

class BaseFileAdmin(admin.ModelAdmin):
    change_list_template = 'template_editor/change_list.html'
    change_form_template = 'template_editor/change_form.html'

    def get_urls(self):
        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name
        
        urlpatterns = [
            url(r'^version/(?P<id>.+)/removeConflict/$', 
                wrap(self.removeConflictView), 
                name="%s_%s_removeConflict" % info),
        ] + super(BaseFileAdmin, self).get_urls()

        return urlpatterns

    def removeConflictView(self, request, extra_context=None, **kwargs):
        version = Version.objects.get(id=kwargs['id'])
        version.conflict = False
        version.save()

        info = self.model._meta.app_label, self.model._meta.model_name
        url = reverse('admin:%s_%s_changelist' % info)
        messages.success(request, _('Conflict solved.'))
        return HttpResponseRedirect(url)


    def get_form(self, request, obj=None, **kwargs):
        if not obj:
            # add Form 新增能夠上傳檔案來新增 static 檔案
            return super(BaseFileAdmin, self).get_form(request, obj, **kwargs)

        # is folder
        if obj.type == FOLDER:
            return FolderFrom

        # is file
        versionNum = request.GET.get('version', None)
        if versionNum:
            version = obj.getVersion(versionNumber=versionNum)
        else:
            version = obj.version

        fullPath = version.absoluteBackupPath
        subFileName = obj.subFileName

        if subFileName in TEXT_SUB_FILE_NAME:
            form = TextFrom
            with open(fullPath, 'rb') as f:
                data = f.read()
            form.base_fields['content'].initial = data
        elif subFileName in IMAGE_SUB_FILE_NAME:
            form = ImageFrom
        else:
            form = FolderFrom

        return form

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = {}
        obj = self.model.objects.get(id=object_id)

        versionNum = request.GET.get('version', None)
        if versionNum:
            extra_context['version'] = obj.getVersion(versionNumber=versionNum)
        else:
            extra_context['version'] = obj.version

        subFileName = obj.subFileName

        if subFileName in TEXT_SUB_FILE_NAME:
            extra_context['isImg'] = False
        elif subFileName in IMAGE_SUB_FILE_NAME:
            extra_context['isImg'] = True
        else:
            extra_context['isImg'] = False

        extra_context['moduleName'] = obj._meta.verbose_name_plural

        return super(BaseFileAdmin, self).change_view(
            request, object_id, form_url=form_url, extra_context=extra_context)

    def response_change(self, request, obj):
        versionNum = request.GET.get('version', None)
        if versionNum:
            version = obj.getVersion(versionNumber=versionNum)
        else:
            version = obj.version

        if obj.type == FILE:
            subFileName = obj.subFileName

            if subFileName in TEXT_SUB_FILE_NAME:
                # 能支援上傳修改檔案
                content = request.POST.get('content')
                
                fullPath = obj.absolutePath
                with open(fullPath, 'wb') as f:
                    data = f.write(content.encode('utf-8'))
            elif subFileName in IMAGE_SUB_FILE_NAME:
                # 圖片轉檔
                image = request.FILES.get('image')
                fullPath = obj.absolutePath

                if image:
                    data = image.read()
                    image.close()

                    with open(fullPath, 'wb') as f:
                        data = f.write(data)
                elif not obj.version == version:
                    backupPath = version.absoluteBackupPath
                    with open(backupPath, 'rb') as f:
                        data = f.read()
                    fullPath = obj.absolutePath
                    with open(fullPath, 'wb') as f:
                        data = f.write(data)
                else:
                    pass
            
            version = createVersion(self.model, obj)

        return super(BaseFileAdmin, self).response_change(request, obj)

    def changelist_view(self, request, extra_context=None):
        conflictNodeList = checkVersion(self.model)

        roots = self.model.objects.filter(level=0)
         
        extra_context = {
            'roots': roots,
            'conflictNodeList': conflictNodeList,
        }
        
        return super(BaseFileAdmin, self).changelist_view(
            request,  extra_context=extra_context)

class StaticAdmin(BaseFileAdmin):
    pass

class TemplateAdmin(BaseFileAdmin):
    pass

admin.site.register(Static, StaticAdmin)
admin.site.register(Template, TemplateAdmin)
